package net.nourdine.jp.build;

import java.util.List;
import net.nourdine.jp.config.ConfigReader;
import net.nourdine.jp.dependency.DependencyManager;
import net.nourdine.jp.io.FileContentManager;
import net.nourdine.jp.log.JsPackerLogger;
import java.io.File;
import java.text.MessageFormat;
import java.util.Properties;
import net.nourdine.jp.dialog.UIDialog;

public class JsProject {

   private static JsPackerLogger log = JsPackerLogger.getInstance();
   public static Properties dialog = UIDialog.getInstace().getProperties();
   private boolean minify;
   private boolean verbose;
   private ConfigReader confReader;
   private String projectName;

   public JsProject(String configFile, boolean minify, boolean verbose) {
      confReader = new ConfigReader(configFile);
      confReader.checkConfigFileAgainstDTD(); // remember to call this for validation!
      projectName = confReader.getProjectName();
      this.minify = minify;
      this.verbose = verbose;
   }

   public void buildStandalone() {
      String code = build();
      String pathToBuildFile = confReader.getTargetFolder() + File.separatorChar + confReader.getBuildFileName();
      FileContentManager.setContent(pathToBuildFile, code);
   }

   public String buildAsDependency() {
      return build();
   }

   private boolean isSelfDependent(String dependency) {
      return dependency.equals(confReader.getXmlConfigPhysicalPath());
   }

   private String getDependencyCode() {
      String code = "";
      if (confReader.hasDependencies()) {
         log.info(MessageFormat.format(dialog.getProperty("dependencies_begin"), projectName));
         for (String dep : confReader.getDependencies()) {
            if (!isSelfDependent(dep)) {
               if (!DependencyManager.isDependencyAlreadyImported(dep)) {
                  DependencyManager.addDependency(dep);
                  code += (new JsProject(dep, minify, verbose)).buildAsDependency();
                  log.info(MessageFormat.format(dialog.getProperty("dependency_included"), projectName, dep));
               } else {
                  log.info(MessageFormat.format(dialog.getProperty("dependency_skipped"), projectName, dep));
               }
            } else {
               log.info(projectName + " >> This project depends on itself! Toooooo bad man!");
            }
         }
      } else {
         log.info(MessageFormat.format(dialog.getProperty("no_dependencies"), projectName));
      }
      log.info(MessageFormat.format(dialog.getProperty("dependencies_end"), projectName));
      return code;
   }

   private String build() {
      // add the lib itself so that no1 will ever import it again!
      DependencyManager.addDependency(confReader.getXmlConfigPhysicalPath());
      String code = "";
      code += getDependencyCode();
      List<String> filesToBuild = confReader.getJsFiles();
      JsFileSet builder = new JsFileSet(filesToBuild);
      code += builder.build(minify);
      return code;
   }
}
